@@ -55,10 +55,9 @@
 #' functions.
 #'
 #' @param .env An environment.
-#' @param ... Pairs of names and expressions, values or functions.
-#'   Pass [zap()] objects to remove bindings.
-#'
-#'   These dots support [tidy dots][tidy-dots] features.
+#' @param ... <[dynamic][dyn-dots]> Named objects (`env_bind()`),
+#'   expressions `env_bind_lazy()`, or functions (`env_bind_active()`).
+#'   Use [zap()] to remove bindings.
 #' @return The input object `.env`, with its associated environment
 #'   modified in place, invisibly.
 #' @export
@@ -112,7 +111,7 @@ env_bind <- function(.env, ...) {
 }
 env_bind_impl <- function(env, data, fn, bind = FALSE, binder = NULL) {
   if (!is_vector(data)) {
-    type <- as_friendly_type(type_of(data))
+    type <- friendly_type_of(data)
     abort(sprintf("`data` must be a vector not a %s", type))
   }
   if (length(data)) {
@@ -125,34 +124,35 @@ env_bind_impl <- function(env, data, fn, bind = FALSE, binder = NULL) {
     }
   }
 
-  env_ <- get_env_retired(env, fn, caller_env(2))
+  env <- get_env_retired(env, fn)
+  data <- vec_coerce(data, "list")
   nms <- names2(data)
 
   if (bind) {
     old <- new_list(length(nms), nms)
-    overwritten <- env_has(env_, nms)
-    old[overwritten] <- env_get_list(env_, nms[overwritten])
+    overwritten <- env_has(env, nms)
+    old[overwritten] <- env_get_list(env, nms[overwritten])
     old[!overwritten] <- list(zap())
+
+    # We don't allow duplicates so we can remove bindings separately
+    zapped <- map_lgl(data, is_zap) & overwritten
+    rm(list = nms[zapped], envir = env)
+    data <- data[!zapped]
+    nms <- names(data) %||% chr()
   }
 
   if (is_null(binder)) {
-    binder <- function(env, nm, value) {
-      base::assign(nm, data[[nm]], envir = env)
-    }
-  }
-
-  for (i in seq_along(data)) {
-    if (bind && overwritten[[i]] && is_zap(maybe_missing(data[[i]]))) {
-      base::rm(list = nms[[i]], envir = env)
-    } else {
-      binder(env_, nms[[i]], data[[i]])
+    .Call(rlang_env_bind_list, env, nms, data);
+  } else {
+    for (i in seq_along(data)) {
+      binder(env, nms[[i]], data[[i]])
     }
   }
 
   if (bind) {
     invisible(old)
   } else {
-    invisible(env_)
+    invisible(env)
   }
 }
 
@@ -257,14 +257,14 @@ env_bind_active <- function(.env, ...) {
 #'
 #' @description
 #'
-#' * `scoped_bindings()` temporarily changes bindings in `.env` (which
+#' * `local_bindings()` temporarily changes bindings in `.env` (which
 #'   is by default the caller environment). The bindings are reset to
 #'   their original values when the current frame (or an arbitrary one
 #'   if you specify `.frame`) goes out of scope.
 #'
 #' * `with_bindings()` evaluates `expr` with temporary bindings. When
 #'   `with_bindings()` returns, bindings are reset to their original
-#'   values. It is a simple wrapper around `scoped_bindings()`.
+#'   values. It is a simple wrapper around `local_bindings()`.
 #'
 #' @inheritParams env_bind
 #' @param ... Pairs of names and values. These dots support splicing
@@ -272,7 +272,7 @@ env_bind_active <- function(.env, ...) {
 #' @param .frame The frame environment that determines the scope of
 #'   the temporary bindings. When that frame is popped from the call
 #'   stack, bindings are switched back to their original values.
-#' @return `scoped_bindings()` returns the values of old bindings
+#' @return `local_bindings()` returns the values of old bindings
 #'   invisibly; `with_bindings()` returns the value of `expr`.
 #' @export
 #' @examples
@@ -282,25 +282,25 @@ env_bind_active <- function(.env, ...) {
 #' # `foo` will be temporarily rebinded while executing `expr`
 #' with_bindings(paste(foo, bar), foo = "rebinded")
 #' paste(foo, bar)
-scoped_bindings <- function(..., .env = .frame, .frame = caller_env()) {
-  env <- get_env_retired(.env, "scoped_bindings()")
+local_bindings <- function(..., .env = .frame, .frame = caller_env()) {
+  env <- get_env_retired(.env, "local_bindings()")
 
   bindings <- list2(...)
   if (!length(bindings)) {
     return(list())
   }
 
-  old <- env_bind_impl(env, bindings, "scoped_bindings()", bind = TRUE)
-  scoped_exit(frame = .frame, !!call2(env_bind_impl, env, old, bind = TRUE))
+  old <- env_bind_impl(env, bindings, "local_bindings()", bind = TRUE)
+  local_exit(frame = .frame, !!call2(env_bind_impl, env, old, bind = TRUE))
 
   invisible(old)
 }
-#' @rdname scoped_bindings
+#' @rdname local_bindings
 #' @param .expr An expression to evaluate with temporary bindings.
 #' @export
 with_bindings <- function(.expr, ..., .env = caller_env()) {
   env <- get_env_retired(.env, "with_bindings()")
-  scoped_bindings(..., .env = .env)
+  local_bindings(..., .env = .env)
   .expr
 }
 
@@ -316,6 +316,8 @@ with_bindings <- function(.expr, ..., .env = caller_env()) {
 #' @return A copy of `.env` enclosing the new environment containing
 #'   bindings to `...` arguments.
 #' @seealso [env_bind()], [env_unbind()]
+#'
+#' @keywords internal
 #' @export
 #' @examples
 #' orig_env <- env(a = 10)
@@ -352,7 +354,7 @@ env_bury <- function(.env, ...) {
 #'   modified in place, invisibly.
 #' @export
 #' @examples
-#' data <- set_names(as_list(letters), letters)
+#' data <- set_names(as.list(letters), letters)
 #' env_bind(environment(), !!! data)
 #' env_has(environment(), letters)
 #'
@@ -555,16 +557,16 @@ scope_poke <- function(env, nm, value, create) {
 #' @section Encoding:
 #'
 #' There are deep encoding issues when you convert a string to symbol
-#' and vice versa. Symbols are _always_ in the native encoding (see
-#' [set_chr_encoding()]). If that encoding (let's say latin1) cannot
-#' support some characters, these characters are serialised to
-#' ASCII. That's why you sometimes see strings looking like
-#' `<U+1234>`, especially if you're running Windows (as R doesn't
-#' support UTF-8 as native encoding on that platform).
+#' and vice versa. Symbols are _always_ in the native encoding. If
+#' that encoding (let's say latin1) cannot support some characters,
+#' these characters are serialised to ASCII. That's why you sometimes
+#' see strings looking like `<U+1234>`, especially if you're running
+#' Windows (as R doesn't support UTF-8 as native encoding on that
+#' platform).
 #'
 #' To alleviate some of the encoding pain, `env_names()` always
 #' returns a UTF-8 character vector (which is fine even on Windows)
-#' with unicode points unserialised.
+#' with ASCII unicode points translated back to UTF-8.
 #'
 #' @inheritParams get_env
 #' @return A character vector of object names.
