@@ -1,8 +1,13 @@
 #include "internal/arg.c"
+#include "internal/call.c"
 #include "internal/dots.c"
+#include "internal/env.c"
+#include "internal/env-binding.c"
+#include "internal/eval.c"
 #include "internal/eval-tidy.c"
 #include "internal/expr-interp.c"
 #include "internal/expr-interp-rotate.c"
+#include "internal/fn.c"
 #include "internal/internal.c"
 #include "internal/quo.c"
 #include "internal/utils.c"
