@@ -4,14 +4,14 @@ test_that("can't add an exit event at top-level", {
   expect_true(TRUE)
   # This can only be tested interactively
   if (FALSE) {
-    scoped_exit(1)  # Can't add an exit event at top level
+    local_exit(1)  # Can't add an exit event at top level
   }
 })
 
 test_that("can add an exit event within a non-top-level global frame", {
   local(envir = global_env(), {
     `_x` <- list()
-    rlang:::scoped_exit(`_x` <- c(`_x`, "bar"))
+    rlang:::local_exit(`_x` <- c(`_x`, "bar"))
     `_x` <- c(`_x`, "foo")
   })
 
