@@ -4,9 +4,9 @@ context("trace.R")
 test_that("tree printing only changes deliberately", {
   skip_unless_utf8()
 
-  scoped_options(
+  local_options(
     rlang_trace_format_srcrefs = TRUE,
-    rlang_trace__force_dangling_srcrefs = TRUE
+    `rlang:::trace_force_dangling_srcrefs` = TRUE
   )
 
   dir <- normalizePath(test_path(".."))
@@ -34,9 +34,9 @@ test_that("can print tree with collapsed branches", {
   # Fake eval() call does not have same signature on old R
   skip_if(getRversion() < "3.4")
 
-  scoped_options(
+  local_options(
     rlang_trace_format_srcrefs = TRUE,
-    rlang_trace__force_dangling_srcrefs = TRUE
+    `rlang:::trace_force_dangling_srcrefs` = TRUE
   )
 
   dir <- normalizePath(test_path(".."))
@@ -103,7 +103,7 @@ test_that("cli_branch() handles edge case", {
 })
 
 test_that("trace formatting picks up `rlang_trace_format_srcrefs`", {
-  scoped_options(rlang_trace__force_dangling_srcrefs = TRUE)
+  local_options(`rlang:::trace_force_dangling_srcrefs` = TRUE)
 
   e <- environment()
   f <- function() trace_back(e)
@@ -388,7 +388,7 @@ test_that("collapsing of eval() frames detects when error occurs within eval()",
   trace <- NULL
 
   fn <- function() {
-    scoped_options(
+    local_options(
       rlang_trace_format_srcrefs = FALSE
     )
     e <<- current_env()
@@ -509,7 +509,7 @@ test_that("can trim layers of backtraces", {
   trace3 <- f(3)
 
   expect_known_output(file = test_path("test-trace-trim.txt"), {
-    scoped_options(rlang_trace_format_srcrefs = FALSE)
+    local_options(rlang_trace_format_srcrefs = FALSE)
 
     cat_line("No trimming:")
     summary(trace0)
@@ -614,8 +614,9 @@ test_that("fails when `bottom` is not on the stack", {
 
 test_that("caught error does not display backtrace in knitted files", {
   skip_if(!rmarkdown::pandoc_available())
+  skip_if_not_installed("knitr")
 
-  scoped_options(
+  local_options(
     rlang_backtrace_on_error = NULL,
     rlang_interactive = FALSE
   )
@@ -623,3 +624,37 @@ test_that("caught error does not display backtrace in knitted files", {
   error_line <- lines[[length(lines)]]
   expect_match(error_line, "foo$")
 })
+
+test_that("empty backtraces are dealt with", {
+  foo <- NULL
+
+  local({
+    env <- new.env()
+    local_options(rlang_trace_top_env = env)
+    tryCatch(
+      error = identity,
+      withCallingHandlers(
+        error = function(cnd) foo <<- cnd_entrace(cnd),
+        eval(quote(stop("stop")), env)
+      )
+    )
+  })
+
+  expect_identical(trace_length(foo$trace), 0L)
+})
+
+test_that("can trace back with quosured symbol", {
+  e <- current_env()
+  f <- function(foo = g()) {
+    # This will create a call in the call stack that isn't really a call
+    quo <- quo(foo)
+
+    # Quosure must be nested otherwise `eval_tidy()` unwraps it
+    eval_tidy(expr(identity(!!quo)))
+  }
+  g <- function() trace_back(e)
+
+  # FIXME: Weird trace structure
+  trace <- f()
+  expect_is(trace, "rlang_trace")
+})
